package dacSketches.ejbDynamicBinding.client;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import dacSketches.ejbDynamicBinding.ejb.common.SimpleTrace;
import dacSketches.ejbDynamicBinding.ejb.dynamicLiaison.LiaisonInterface;


public class DynamicLiaisonClient
	extends SimpleTrace
{
	private String					objName;
	private Context					env;

	private LiaisonInterface		lcProxy = null;
	private LiaisonInterface		ucProxy = null;

	private final String lcJNDIKey = new String( "lcJNDI" );
	private final String ucJNDIKey = new String( "ucJNDI" );
	private final String lcHomeKey = new String( "lcHomeClass" );
	private final String ucHomeKey = new String( "ucHomeClass" );

	protected DynamicLiaisonClient()
		throws   NamingException
			   , ClassNotFoundException
			   , NoSuchMethodException
			   , IllegalAccessException
			   , InvocationTargetException
	{
		super();

		objName = this.getClass().getName();
		Context initial = new InitialContext();
		env = (Context) initial.lookup( "java:comp/env" );

		lcProxy = acquire( lcJNDIKey, lcHomeKey );
		ucProxy = acquire( ucJNDIKey, ucHomeKey );
	}

	protected LiaisonInterface acquire( String jndiKey, String homeKey )
		throws   NamingException
			   , ClassNotFoundException
			   , NoSuchMethodException
			   , IllegalAccessException
			   , InvocationTargetException
	{
		String	mName = new String( "connect" );
		log( objName, mName, "Invoked" );

		try
		{
			String jndi = (String) env.lookup( jndiKey );
			String homeName = (String) env.lookup( homeKey );

			log( objName, mName, "jndi = " + jndi );
			log( objName, mName, "homeName = " + homeName );

			//	Throws ClassNotFoundException...
			Class homeClass = Class.forName( homeName );

			log( objName, mName, "Preparing to acquire home references." );

			Object ref = env.lookup( "ejb/" + jndi );
			Object home = PortableRemoteObject.narrow( ref, homeClass );

			//	Throws NoSuchMethodException and SecurityException...
			Method createRemoteProxy = homeClass.getMethod( "create", null );

			log( objName, mName, "Preparing to create the remote proxy." );

			//	Throws IllegalAccessException, IllegalArgumentException,
			//	InvocationTargetException, NullPointerException,
			//	and ExceptionInitializerError...
			LiaisonInterface proxy = (LiaisonInterface) createRemoteProxy.invoke( home, null );

			log( objName, mName, "Created the remote object." );

			return proxy;
		}
		finally
		{
			log( objName, mName, "Finished" );
		}
	}

	protected void runDemo()
		throws RemoteException
	{
		String	mName = new String( "runDemo" );
		log( objName, mName, "Invoked" );

		try
		{
			String testString = new String( "The Quick Brown Fox..." );
			String lcString = (String) lcProxy.forward( testString );
			String ucString = (String) ucProxy.forward( testString );

			System.out.println( "testString = " + testString );
			System.out.println( "lower case converstion = " + lcString );
			System.out.println( "upper case converstion = " + ucString );
		}
		finally
		{
			log( objName, mName, "Finished" );
		}
	}

	public static void main( String[] args )
	{
		try
		{
			DynamicLiaisonClient self = new DynamicLiaisonClient();
			self.runDemo();
			System.exit( 0 );
		}
		catch (Exception ex)
		{
			System.err.println( "Caught an unexpected exception!" );
			ex.printStackTrace();
		}
	}
}

