package dacSketches.ejbDynamicBinding.ejb.dynamicLiaison;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.CreateException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.rmi.PortableRemoteObject;

import dacSketches.ejbDynamicBinding.ejb.common.SessionAdapter;

/*
 *	A stateless session bean that demonstrates dynamic binding.
 */

public class DynamicLiaisonBean
	extends SessionAdapter
	implements LiaisonInterface
{
	private String					beanName;
	private String					targetJNDI;
	private String					converterHomeName;

	private ConverterInterface		proxy = null;

	private final String beanNameKey = new String( "beanName" );
	private final String targetJNDIKey = new String( "targetJNDI" );
	private final String converterHomeKey = new String( "targetHomeClass" );


	//	Method(s) defined in LiaisonInterface.
	public String forward( String obj )
		throws RemoteException
	{
		String myMethod = "forward";
		log( beanName, myMethod, "Invoked" );

		try
		{
			return proxy.process( obj );
		}
		finally
		{
			log( beanName, myMethod, "Finished" );
		}
	}
		
	//  Methods defined in SessionBean.

	public void ejbCreate()
		throws CreateException
	{
		super.ejbCreate();
		
		String myMethod = "ejbCreate";
		log( beanName, myMethod, "Invoked" );

		try
		{
			Context ictx = new InitialContext();
			Context env = (Context) ictx.lookup( "java:comp/env" );

			beanName = (String) env.lookup( beanNameKey );
			targetJNDI = (String) env.lookup( targetJNDIKey );
			converterHomeName = (String) env.lookup( converterHomeKey );

			log( beanName, myMethod, "targetJNDI = " + targetJNDI );
			log( beanName, myMethod, "converterHomeName = " + converterHomeName );

			//	Throws ClassNotFoundException...
			Class converterHome = Class.forName( converterHomeName );

			log( beanName, myMethod, "Preparing to acquire the home reference." );

			Object ref = env.lookup( "ejb/" + targetJNDI );
			Object home = PortableRemoteObject.narrow( ref, converterHome );

			//	Throws NoSuchMethodException and SecurityException...
			Method createRemoteProxy = converterHome.getMethod( "create", null );

			log( beanName, myMethod, "Preparing to create the remote proxy." );

			//	Throws IllegalAccessException, IllegalArgumentException,
			//	InvocationTargetException, NullPointerException,
			//	and ExceptionInitializerError...
			proxy = (ConverterInterface) createRemoteProxy.invoke( home, null );

			log( beanName, myMethod, "Created the remote object." );
		}
		catch (NamingException e)
		{
			log( beanName, myMethod, e.getMessage() );
			throw new CreateException( e.getMessage() );
		}
		catch (ClassNotFoundException e)
		{
			log( beanName, myMethod, e.getMessage() );
			throw new CreateException( e.getMessage() );
		}
		catch (NoSuchMethodException e)
		{
			log( beanName, myMethod, e.getMessage() );
			throw new CreateException( e.getMessage() );
		}
		catch (IllegalAccessException e)
		{
			log( beanName, myMethod, e.getMessage() );
			throw new CreateException( e.getMessage() );
		}
		catch (InvocationTargetException e)
		{
			log( beanName, myMethod, e.getMessage() );
			throw new CreateException( e.getMessage() );
		}
		finally
		{
			log( beanName, myMethod, "Finished" );
		}
	}

	public void ejbRemove()
	{
		String myMethod = "ejbRemove";
		log( beanName, myMethod, "Invoked" );
		log( beanName, myMethod, "Finished" );
	}
}

